import { inspect } from 'util';
import { SyntaxKind } from 'typescript';
const found = [];
function showKind(kind) {
    return {
        [inspect.custom](_, options) {
            return '' + inspect(kind, options) + ' \x1B[38;5;4m[SyntaxKind.' + SyntaxKind[kind] + ']\x1B[0m';
        },
    };
}
function deepRemoveParent(node) {
    if (found.includes(node)) {
        const id = found.indexOf(node) + 1;
        return [
            {
                [inspect.custom](_, options) {
                    return `\x1B[38;5;7mCircular\x1B[0m <OBJ:${inspect(id, options)}>`;
                },
            },
            true,
        ];
    }
    const id = found.push(node);
    const clone = { ...node };
    let change = false;
    for (const k in clone) {
        if (Array.isArray(clone[k])) {
            clone[k] = clone[k].map((e) => deepRemoveParent(e)[0]);
            continue;
        }
        if (k === 'parent') {
            change = true;
            if (clone[k] && clone[k].kind) {
                clone['parent'] = showKind(clone[k].kind);
            }
            else {
                delete clone['parent'];
            }
        }
        else if (k === 'kind') {
            clone[k] = showKind(clone[k]);
            change = true;
        }
        else if (typeof clone[k] === 'object' && clone[k]) {
            const [repl, mchange] = deepRemoveParent(clone[k]);
            if (mchange) {
                clone[k] = repl;
                change = true;
            }
        }
    }
    let ret;
    if (change) {
        ret = [clone, true];
    }
    else {
        ret = [node, false];
    }
    ret[0][inspect.custom] = function (_, options) {
        const { [inspect.custom]: x, ...node } = this;
        return `<OBJ:${inspect(id, options)}> ` + inspect(node, { ...options });
    };
    return ret;
}
export function dumpFlagStrings(flags, def, sp = '\n\t') {
    const hit = new Set();
    for (const item of Object.values(def)) {
        if (item === 0) {
            if (flags === 0) {
                hit.add(`${def[item]}(${item})`);
            }
        }
        else if ((item & flags) === item) {
            hit.add(`${def[item]}(${item})`);
        }
    }
    return inspect(flags, { colors: true }) + ' =\x1B[38;5;6m' + [...hit.values()].join(sp) + '\x1B[0m';
}
export function dumpFlags(flags, def) {
    console.error(dumpFlagStrings(flags, def));
}
export function prettyKind(node) {
    return '{ ' + inspect(showKind(node.kind)) + ' }';
}
export function dumpNode(node, options = {}) {
    found.length = 0;
    const v = Array.isArray(node) ? node.map((e) => deepRemoveParent(e)[0]) : deepRemoveParent(node)[0];
    console.log(inspect(v, {
        depth: 10,
        colors: true,
        compact: false,
        customInspect: true,
        ...options,
    }));
    found.length = 0;
}
//# sourceMappingURL=dumpNode.js.map