"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveProjectFile = void 0;
const path_1 = require("path");
const fs_extra_1 = require("fs-extra");
function resolveProjectFile(node, program) {
    const importPath = node.moduleSpecifier.text;
    const currentFile = node.getSourceFile().fileName;
    const wantFile = path_1.resolve(currentFile, '..', importPath);
    let checkExtensions = ['.ts', '.tsx', path_1.extname(currentFile)];
    if (program) {
        if (program.getCompilerOptions().allowJs) {
            checkExtensions.push('.js', '.jsx');
        }
    }
    for (const ext of checkExtensions) {
        if (fs_extra_1.pathExistsSync(wantFile + ext)) {
            return {
                type: "typescript" /* typescript */,
                sourceKind: "internal" /* internal */,
                nodeResolve: importPath,
                fsPath: wantFile + ext,
                specifier: importPath,
            };
        }
    }
    return {
        type: "missing" /* missing */,
        specifier: importPath,
    };
}
exports.resolveProjectFile = resolveProjectFile;
//# sourceMappingURL=resolve.tsprogram.js.map